;;; bandali-erc.el --- bandali's ERC setup           -*- lexical-binding: t; -*-

;; Copyright (C) 2018-2020  Amin Bandali

;; Author: Amin Bandali <bandali@gnu.org>
;; Keywords: tools

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

;;; Commentary:

;; My ERC setup for IRC.  It uses my fork of ZNC.el.

;;; Code:

(with-eval-after-load 'erc
  (csetq
   erc-auto-query 'bury
   erc-autojoin-domain-only nil
   erc-format-nick-function 'erc-format-@nick
   erc-join-buffer 'bury
   erc-lurker-hide-list '("JOIN" "PART" "QUIT")
   erc-nick "bandali"
   erc-prompt "erc>"
   erc-rename-buffers t
   erc-server-reconnect-attempts 5
   erc-server-reconnect-timeout 3)

  (declare-function erc-message "erc-backend"
                    (message-command line &optional force))
  (declare-function erc-default-target "erc")
  (declare-function erc-current-nick "erc")
  (defun erc-cmd-OPME ()
    "Request chanserv to op me."
    (erc-message "PRIVMSG"
                 (format "chanserv op %s %s"
                         (erc-default-target)
                         (erc-current-nick)) nil))
  (declare-function erc-cmd-DEOP "erc" (&rest people))
  (defun erc-cmd-DEOPME ()
    "Deop myself from current channel."
    (erc-cmd-DEOP (format "%s" (erc-current-nick))))
  (add-to-list 'erc-modules 'keep-place)
  (add-to-list 'erc-modules 'notifications)
  (add-to-list 'erc-modules 'smiley)
  (add-to-list 'erc-modules 'spelling)
  (declare-function erc-update-modules "erc")
  (erc-update-modules)

  ;; erc-fill
  (csetq
   erc-fill-column 77
   erc-fill-function 'erc-fill-static
   erc-fill-static-center 18)

  ;; erc-match
  ;; (csetq
  ;;  erc-pal-highlight-type 'message
  ;;  erc-pals
  ;;  '("aindilis" "blackbeard" "bremner" "brettgilio" "civodul" "dto"
  ;;    "ggoes" "jrasata" "jxself" "mplsCorwin" "quiliro" "rindolf" "rwp"
  ;;    "sachac" "technomancy"))
  ;; (with-eval-after-load 'erc-match
  ;;   (set-face-attribute
  ;;    'erc-pal-face nil
  ;;    :foreground 'unspecified
  ;;    :weight 'unspecified
  ;;    ;; :inherit 'font-lock-string-face
  ;;    ;; :background (face-attribute 'font-lock-string-face :background)
  ;;    :background "#ffffed"))

  ;; erc-pcomplete
  (csetq erc-pcomplete-nick-postfix ", ")

  ;; erc-track
  (csetq
   erc-track-enable-keybindings nil
   erc-track-exclude-types '("JOIN" "MODE" "NICK" "PART" "QUIT"
                             "324" "329" "332" "333" "353" "477")
   erc-track-position-in-mode-line t
   erc-track-priority-faces-only 'all
   erc-track-shorten-function nil
   erc-track-showcount t)

  ;; key bindings
  (global-set-key (kbd "C-c w e") #'erc-switch-to-buffer-other-window)
  (define-key erc-mode-map (kbd "M-a") #'erc-track-switch-buffer))

(with-eval-after-load 'znc
  (let ((pwd (let ((auth (auth-source-search :host "znca")))
               (cond
                ((null auth) (error "Couldn't find znca's authinfo"))
                (t (funcall (plist-get (car auth) :secret)))))))
    (csetq znc-servers
           `(("znc.emacsconf.org" 6697 t
              ((freenode "bandali/freenode" ,pwd)))
             ("znc.emacsconf.org" 6697 t
              ((oftc "bandali/oftc" ,pwd)))
             ("znc.emacsconf.org" 6697 t
              ((tildechat "bandali/tildechat" ,pwd)))))))
;; global key bindings
(global-set-key (kbd "C-c a e e") #'znc-erc)
(global-set-key (kbd "C-c a e a") #'znc-all)

(provide 'bandali-erc)
;;; bandali-erc.el ends here
